@@ -160,7 +160,8 @@ void allpairs_output_results(int hit_count,
                                   query_head,
                                   qsequence,
                                   qseqlen,
-                                  qsequence_rc);
+                                  qsequence_rc,
+                                  hp->target);
               
           if (fp_userout)
             results_show_userout_one(fp_userout,
@@ -187,7 +188,8 @@ void allpairs_output_results(int hit_count,
                             query_head,
                             qsequence,
                             qseqlen,
-                            qsequence_rc);
+                            qsequence_rc,
+                            0);
       
       if (fp_userout)
         results_show_userout_one(fp_userout,
