@@ -198,7 +198,8 @@ void derep_fulllength()
   /* alloc and init table of links to other sequences in cluster */
 
   unsigned int * nextseqtab = (unsigned int*) xmalloc(sizeof(unsigned int) * dbsequencecount);
-  memset(nextseqtab, 0, sizeof(unsigned int) * dbsequencecount);
+  const unsigned int terminal = (unsigned int)(-1);
+  memset(nextseqtab, -1, sizeof(unsigned int) * dbsequencecount);
 
   char * match_strand = (char *) xmalloc(dbsequencecount);
   memset(match_strand, 0, dbsequencecount);
@@ -418,8 +419,8 @@ void derep_fulllength()
           fprintf(fp_uc, "S\t%ld\t%ld\t*\t*\t*\t*\t*\t%s\t*\n",
                   i, len, h);
           
-          for (unsigned long next = nextseqtab[bp->seqno_first];
-               next;
+          for (unsigned int next = nextseqtab[bp->seqno_first];
+               next != terminal;
                next = nextseqtab[next])
             fprintf(fp_uc,
                     "H\t%ld\t%ld\t%.1f\t%s\t0\t0\t*\t%s\t%s\n",
@@ -519,7 +520,8 @@ void derep_prefix()
   /* alloc and init table of links to other sequences in cluster */
 
   unsigned int * nextseqtab = (unsigned int*) xmalloc(sizeof(unsigned int) * dbsequencecount);
-  memset(nextseqtab, 0, sizeof(unsigned int) * dbsequencecount);
+  const unsigned int terminal = (unsigned int)(-1);
+  memset(nextseqtab, -1, sizeof(unsigned int) * dbsequencecount);
 
   char * seq_up = (char*) xmalloc(db_getlongestsequence() + 1);
 
@@ -565,18 +567,19 @@ void derep_prefix()
       /* compute hashes of all prefixes */
 
       unsigned long fnv1a_hash = 14695981039346656037UL;
+      prefix_hashes[0] = fnv1a_hash;
       for(unsigned int j = 0; j < seqlen; j++)
         {
           fnv1a_hash ^= seq_up[j];
           fnv1a_hash *= 1099511628211UL;
-          prefix_hashes[j] = fnv1a_hash;
+          prefix_hashes[j+1] = fnv1a_hash;
         }
 
       /* first, look for an identical match */
 
       unsigned int prefix_len = seqlen;
 
-      unsigned long hash = prefix_hashes[prefix_len-1];
+      unsigned long hash = prefix_hashes[prefix_len];
       struct bucket * bp = hashtable + (hash & hash_mask);
       
       while ((bp->size) &&
@@ -611,9 +614,10 @@ void derep_prefix()
         {
           /* look for prefix match */
           
-          while((! bp->size) && (prefix_len-- >= len_shortest))
+          while((! bp->size) && (prefix_len > len_shortest))
             {
-              hash = prefix_hashes[prefix_len-1];
+              prefix_len--;
+              hash = prefix_hashes[prefix_len];
               bp = hashtable + (hash & hash_mask);
               
               while ((bp->size) &&
@@ -630,7 +634,7 @@ void derep_prefix()
                 }
             }
           
-          if ((bp->size) && (prefix_len >= len_shortest))
+          if (bp->size)
             {
               /* prefix match */
 
@@ -763,8 +767,8 @@ void derep_prefix()
           fprintf(fp_uc, "S\t%ld\t%ld\t*\t*\t*\t*\t*\t%s\t*\n",
                   i, len, h);
           
-          for (unsigned long next = nextseqtab[bp->seqno_first];
-               next;
+          for (unsigned int next = nextseqtab[bp->seqno_first];
+               next != terminal;
                next = nextseqtab[next])
             fprintf(fp_uc,
                     "H\t%ld\t%lu\t%.1f\t+\t0\t0\t*\t%s\t%s\n",
