@@ -167,7 +167,8 @@ void results_show_uc_one(FILE * fp,
                          char * query_head,
                          char * qsequence,
                          long qseqlen,
-                         char * rc)
+                         char * rc,
+                         int clusterno)
 {
   /*
     http://www.drive5.com/usearch/manual/ucout.html
@@ -180,7 +181,7 @@ void results_show_uc_one(FILE * fp,
     strand: + or -
     0
     0
-    compressed alignment, e.g. 9I92M14D
+    compressed alignment, e.g. 9I92M14D, or "=" if prefect alignment
     query label
     target label
   */
@@ -192,7 +193,7 @@ void results_show_uc_one(FILE * fp,
 
       fprintf(fp,
               "H\t%d\t%ld\t%.1f\t%c\t0\t0\t%s\t%s\t%s\n",
-              hp->target,
+              clusterno,
               qseqlen,
               hp->id,
               hp->strand ? '-' : '+',
@@ -485,9 +486,13 @@ void results_show_alnout(FILE * fp,
                   0.0);
 
 #if 0
-          fprintf(fp, "%d kmers, %d score, %d gap opens. %s\n",
+          fprintf(fp, "%d kmers, %d score, %d gap opens. %s %s %d %d %d %d %d\n",
                   hp->count, hp->nwscore, hp->nwgaps,
-                  hp->accepted ? "accepted" : "not accepted");
+                  hp->accepted ? "accepted" : "not accepted",
+                  hp->nwalignment, hp->nwalignmentlength,
+                  hp->trim_q_left, hp->trim_q_right,
+                  hp->trim_t_left, hp->trim_t_right
+                  );
 #endif
         }
     }
