@@ -161,33 +161,6 @@ bool search_enough_kmers(struct searchinfo_s * si,
   return (count >= opt_minwordmatches) || (count >= si->kmersamplecount);
 }
 
-inline void topscore_insert(int i, struct searchinfo_s * si)
-{
-  count_t count = si->kmers[i];
-  
-  /* ignore sequences with very few kmer matches */
-
-  if (!search_enough_kmers(si, count))
-    return;
-  
-  unsigned int seqno = dbindex_getmapping(i);
-  unsigned int length = db_getsequencelen(seqno);
-
-  elem_t novel;
-  novel.count = count;
-  novel.seqno = seqno;
-  novel.length = length;
-  
-  minheap_add(si->m, & novel);
-}
-
-void _mm_print_epi8(__m128i x)
-{
-  unsigned char * y = (unsigned char*)&x;
-  for (int i=0; i<16; i++)
-    printf("%s%02x", (i>0?" ":""), y[15-i]);
-}
-
 void search_topscores(struct searchinfo_s * si)
 {
   /*
@@ -228,9 +201,25 @@ void search_topscores(struct searchinfo_s * si)
         }
     }
 
+  int minmatches = MIN(opt_minwordmatches, si->kmersamplecount);
+
   for(int i=0; i < indexed_count; i++)
-    topscore_insert(i, si);
-  
+    {
+      count_t count = si->kmers[i];
+      if (count >= minmatches)
+        {
+          unsigned int seqno = dbindex_getmapping(i);
+          unsigned int length = db_getsequencelen(seqno);
+
+          elem_t novel;
+          novel.count = count;
+          novel.seqno = seqno;
+          novel.length = length;
+
+          minheap_add(si->m, & novel);
+        }
+    }
+
   minheap_sort(si->m);
 }
 
@@ -266,37 +255,51 @@ void align_trim(struct hit * hit)
   /* left trim alignment */
   
   char * p = hit->nwalignment;
-  long run = 1;
-  int scanlength = 0;
-  sscanf(p, "%ld%n", &run, &scanlength);
-  char op = *(p+scanlength);
-  if (op != 'M')
+  char op;
+  long run;
+  if (*p)
     {
-      hit->trim_aln_left = 1 + scanlength;
-      if (op == 'D')
-        hit->trim_q_left = run;
-      else
-        hit->trim_t_left = run;
+      run = 1;
+      int scanlength = 0;
+      sscanf(p, "%ld%n", &run, &scanlength);
+      op = *(p+scanlength);
+      if (op != 'M')
+        {
+          hit->trim_aln_left = 1 + scanlength;
+          if (op == 'D')
+            hit->trim_q_left = run;
+          else
+            hit->trim_t_left = run;
+        }
     }
   
   /* right trim alignment */
   
   char * e = hit->nwalignment + strlen(hit->nwalignment);
-  p = e - 1;
-  op = *p;
-  if (op != 'M')
+  if (e > hit->nwalignment)
     {
-      while (*(p-1) <= '9')
-        p--;
-      run = 1;
-      sscanf(p, "%ld", &run);
-      hit->trim_aln_right = e - p;
-      if (op == 'D')
-        hit->trim_q_right = run;
-      else
-        hit->trim_t_right = run;
+      p = e - 1;
+      op = *p;
+      if (op != 'M')
+        {
+          while ((p > hit->nwalignment) && (*(p-1) <= '9'))
+            p--;
+          run = 1;
+          sscanf(p, "%ld", &run);
+          hit->trim_aln_right = e - p;
+          if (op == 'D')
+            hit->trim_q_right = run;
+          else
+            hit->trim_t_right = run;
+        }
     }
-  
+
+  if (hit->trim_q_left >= hit->nwalignmentlength)
+    hit->trim_q_right = 0;
+
+  if (hit->trim_t_left >= hit->nwalignmentlength)
+    hit->trim_t_right = 0;
+
   hit->internal_alignmentlength = hit->nwalignmentlength
     - hit->trim_q_left - hit->trim_t_left
     - hit->trim_q_right - hit->trim_t_right;
@@ -306,9 +309,9 @@ void align_trim(struct hit * hit)
     - hit->trim_q_right - hit->trim_t_right;
 
   hit->internal_gaps = hit->nwgaps
-    - (hit->trim_q_left  + hit->trim_t_left  > 0 ? 1 : 0)
-    - (hit->trim_q_right + hit->trim_t_right > 0 ? 1 : 0);
-  
+    - ((hit->trim_q_left  + hit->trim_t_left)  > 0 ? 1 : 0)
+    - ((hit->trim_q_right + hit->trim_t_right) > 0 ? 1 : 0);
+
   /* CD-HIT */
   hit->id0 = hit->shortest > 0 ? 100.0 * hit->matches / hit->shortest : 0.0;
   /* all diffs */
